# -*- coding: utf-8 -*-
#
#
# $Id: gnspmsg.py,v 1.3 2004/08/03 07:42:25 hd Exp $

"""Open Global Name Service Protocol

Stability: None

@author: U{HD<mailto:hdcola@gmail.com>}

@see: Open Global Name Service Protocol淶
"""
from uss.message import bytemsg
import struct

class GNSPMessage(bytemsg.ByteMessage):
    """GNSSP Ϣ"""
    def __init__(self):
        # Ϣͷ
        self.head = GNSPMessageHead(self)
        # Ϣ
        self.body = bytemsg.ByteMessageBody(self)
        # Ϣ
        self.msgutilcls = GNSMessageUtil
        # ǰЭ
        self.protocolname = 'gnsp'
        # ǰϢ
        self.msgname = ''
        
    def pack_gnsp_connect(self, fields):
        """connectĴ"""
        return struct.pack('6s16sI8s',
                fields['system_id'],
                fields['auth_source'],
                fields['version'],
                fields['time_stamp']
                )

    def unpack_gnsp_connect(self, packet):
        """connectĽ"""
        self.body.fields['system_id'], \
            self.body.fields['auth_source'], \
            self.body.fields['version'], \
            self.body.fields['time_stamp'] = \
            struct.unpack('6s16sI8s',packet)

    def pack_gnsp_connect_resp(self, fields):
        """connect_respĴ"""
        return struct.pack('II',
                fields['status'],
                fields['version']
                )

    def unpack_gnsp_connect_resp(self, packet):
        """connect_respĽ"""
        self.body.fields['status'], \
            self.body.fields['version'] = \
            struct.unpack('II',packet)

    def pack_gnsp_mail_counter(self, fields):
        """mail_counterĴ"""
        return struct.pack('16s', fields['uid'])

    def unpack_gnsp_mail_counter(self, packet):
        """mail_counterĽ"""
        (self.body.fields['uid'],) = struct.unpack('16s',packet)

    def pack_gnsp_mail_counter_resp(self, fields):
        """mail_counter_respĴ"""
        return struct.pack('16sI',
                fields['uid'],
                fields['number']
                )

    def unpack_gnsp_mail_counter_resp(self, packet):
        """mail_counter_respĽ"""
        self.body.fields['uid'], \
            self.body.fields['number'] = \
            struct.unpack('16sI', packet)

    def pack_gnsp_terminate(self, fields):
        """terminateΪ"""
        return None

    def unpack_gnsp_terminate(self, packet):
        """terminateΪ"""
        return None

    def pack_gnsp_terminate_resp(self, fields):
        """terminate_respΪ"""
        return None

    def unpack_gnsp_terminate_resp(self, packet):
        """terminate_respΪ"""
        return None

class GNSPMessageHead(bytemsg.ByteMessageHead):
    """GNSPϢϢͷ"""
    def __init__(self, parent):
        # 
        self.length = 8
        # ID
        self.id = 0
        # 
        self.parent = parent

    def __len__(self):
        """ͷ8"""
        return 8

    def setId(self,id):
        """ID"""
        self.id = id

    def setLength(self, length):
        """"""
        self.length = length

    def loadHead(self, header):
        """תһPDU (protocol data unit)һϢͷ
        ϢͷĸʽΪlength(4ֽ)commandid4ֽڣ8ֽڳ
        """
        self.length,self.id = struct.unpack('>II', header)
    
    def packed(self):
        """תһϢͷΪ
        ϢͷĸʽΪlength(4ֽ)commandid4ֽڣ12ֽڳ
        """
        return struct.pack('>II', self.length, self.id)

    def __str__(self):
        """ַ"""
        plist = []
        plist.append("ϢID:%s" % self.id)
        plist.append("Ϣȣ%s" % self.length)
        return reduce(lambda x,y: x + "\n" + y, plist)

class GNSMessageUtil(bytemsg.ByteMessageUtil):
    """GNSSPϢ"""
    def __init__(self):
        pass

# GNSSP Message
GNSMessageUtil.commandinfo = {
    0x00000000L : 'generic_noop',        # ʲôı
    0xff000000L : 'generic_noop_resp',   # ʲôıĻظ
    0x00000001L : 'connect',             # 
    0xff000001L : 'connect_resp',        # Ļظ
    0x00000002L : 'terminate',           # Ͽ
    0xff000002L : 'terminate_resp',      # ϿĻظ
    0x00000005L : 'mail_counter',        # ʼͳƱ
    0xff000005L : 'mail_counter_resp'    # ʼͳƱĻظ
    }

# ͨƲϢIDĽṹ
GNSMessageUtil.nametoid = {}
for k in GNSMessageUtil.commandinfo.keys():
    GNSMessageUtil.nametoid[GNSMessageUtil.commandinfo[k]] = k
