# -*- coding: utf-8 -*-
# -*- test-case-name: ussp.test.test_usspmsg -*-
#
#
# $Id: usspmsg.py,v 1.5 2004/08/03 07:42:25 hd Exp $

"""Open Unified Storage System Protocol

Stability: None

@author: U{HD<mailto:hdcola@gmail.com>}

@see: Open Unified Storage System Protocol淶
"""
from uss.message import bytemsg
import struct

class USSPMessage(bytemsg.ByteMessage):
    """USSP Ϣ"""
    def __init__(self):
        # Ϣͷ
        self.head = bytemsg.ByteMessageHead(self)
        # Ϣ
        self.body = bytemsg.ByteMessageBody(self)
        # Ϣ
        self.msgutilcls = USSPMessageUtil
        # Э
        self.protocolname = 'ussp'
        # ǰϢ
        self.msgname = ''

    def pack_ussp_connect(self, fields):
        """connectĴ"""
        return struct.pack('6s16sI8s',
                fields['system_id'],
                fields['auth_source'],
                fields['version'],
                fields['time_stamp']
                )

    def unpack_ussp_connect(self, packet):
        """connectĽ"""
        self.body.fields['system_id'], \
            self.body.fields['auth_source'], \
            self.body.fields['version'], \
            self.body.fields['time_stamp'] = \
            struct.unpack('6s16sI8s',packet)

    def pack_ussp_connect_resp(self, fields):
        """connect_respĴ"""
        return struct.pack('II',
                fields['status'],
                fields['version']
                )

    def unpack_ussp_connect_resp(self, packet):
        """connect_respĽ"""
        self.body.fields['status'], \
            self.body.fields['version'] = \
            struct.unpack('II',packet)

    def pack_ussp_mail_counter(self, fields):
        """mail_counterĴ"""
        return struct.pack('16s', fields['uid'])

    def unpack_ussp_mail_counter(self, packet):
        """mail_counterĽ"""
        (self.body.fields['uid'],) = struct.unpack('16s',packet)

    def pack_ussp_mail_counter_resp(self, fields):
        """mail_counter_respĴ"""
        return struct.pack('16sI',
                fields['uid'],
                fields['number']
                )

    def unpack_ussp_mail_counter_resp(self, packet):
        """mail_counter_respĽ"""
        self.body.fields['uid'], \
            self.body.fields['number'] = \
            struct.unpack('16sI', packet)

    def pack_ussp_terminate(self, fields):
        """terminateΪ"""
        return None

    def unpack_ussp_terminate(self, packet):
        """terminateΪ"""
        pass

    def pack_ussp_terminate_resp(self, fields):
        """terminate_respΪ"""
        return None

    def unpack_ussp_terminate_resp(self, packet):
        """terminate_respΪ"""
        pass

class USSPMessageUtil(bytemsg.ByteMessageUtil):
    """USSPϢ"""
    def __init__(self):
        pass

# USSP Message
USSPMessageUtil.commandinfo = {
    0x00000000L : 'generic_noop',        # ʲôı
    0xff000000L : 'generic_noop_resp',   # ʲôıĻظ
    0x00000001L : 'connect',             # 
    0xff000001L : 'connect_resp',        # Ļظ
    0x00000002L : 'terminate',           # Ͽ
    0xff000002L : 'terminate_resp',      # ϿĻظ
    0x00000005L : 'mail_counter',        # ʼͳƱ
    0xff000005L : 'mail_counter_resp'    # ʼͳƱĻظ
    }

# ͨƲϢIDĽṹ
USSPMessageUtil.nametoid = {}
for k in USSPMessageUtil.commandinfo.keys():
    USSPMessageUtil.nametoid[USSPMessageUtil.commandinfo[k]] = k
